/*************************************************************************************
 Data Visualization - project 2
 Author: Lijuan Cheng
 Supervisor: George Legrady
 
 Interactions: 1. Keyboard control: press c or C to show China, press j or J to show Japan,
 press i or I to show India, press k or K to show Korea and press v or V to show Vietnam.
 2. Press one character to show a country and then press 1 to show the curve lines about the book which tile equals the country name.
 *************************************************************************************/

import peasy.*;
import controlP5.*;

PeasyCam cam;
ControlP5 cp5;

PFont f;
Table table1, table2;
int numRows1, numRows2, numColumns, smallest, largest, year, month, day;
int startYear = 2006;
int endYear = 2021;
int[][] counts, radius;
int buttonWidth = 80;
int buttonHeight =25;
int buttonSpace = 30;
float linePos=0;
boolean showChina, showIndia, showJapan, showKorea, showVietnam, showCurve;

ArrayList<Book> cBooks = new ArrayList<Book>();
ArrayList<Book> jBooks = new ArrayList<Book>();
ArrayList<Book> iBooks = new ArrayList<Book>();
ArrayList<Book> kBooks = new ArrayList<Book>();
ArrayList<Book> vBooks = new ArrayList<Book>();
//ArrayList<Book> books = new ArrayList<Book>();
int[] colors = {#FF0033, #FFFF33, #0000FF, #14e3ce, #c714e3};
String [] months = {"JAN", "FEB", "MAR", "APR", "MAY", "JUN", "JUL", "AUG", "SEP", "OCT", "NOV", "DEC"};
String [] countries = {"China", "India", "Japan", "Korea", "Vietnam"};

void setup() {
  size(1200, 800, P3D);
  //fullScreen(P3D);
  cam = new PeasyCam(this, 200);
  //cp5 = new ControlP5(this);
  showChina = true;
  showIndia = true;
  showJapan = true;
  showKorea = true;
  showVietnam = true;
  showCurve = false;

  f = createFont("Arial", 15);

  //Menu control
  cp5 = new ControlP5(this);

  textFont(f);
  for (int i = 0; i< 5; i++) {
    cp5.addButton(countries[i])
      .setPosition(50, 630+i*buttonSpace)
      .setSize(buttonWidth, buttonHeight)
      .setColorBackground(color(colors[i]))
      .setColorForeground(color(#626262))
      .setColorActive(color(colors[i]))
      .setValue(0)
      .setId(i);
    cp5.getController(countries[i]).getCaptionLabel().toUpperCase(false).setFont(f);
  }
  cp5.setAutoDraw(false);

  table1 = loadTable("FiveAsianCountriesCounts.csv", "header");
  table2 = loadTable("FiveAsianCountriesBooks.csv", "header");

  radiusCalculate(table1, table2);
  booksConstruction(table2);

  smooth(4);
}

void draw() {
  background(0);

  pushMatrix();
  for (int year = 2006; year < 2022; year++) {
    int i = year - 2006;

    //draw the biggest circle to show month information
    stroke(255, 255);
    strokeWeight(0.5);
    noFill();
    float mr = radius[i][0];
    ellipse(0, 0, 1.04*mr, 1.04*mr);
    float inc = TWO_PI/12.0;
    for (int k=0; k < 12; k++) {
      pushMatrix();
      translate(0.5*mr*cos(k*inc), 0.5*mr*sin(k*inc), 0);
      rotateZ(k*inc);
      textAlign(LEFT, CENTER);
      textFont(f, 5);
      fill(255);
      text(months[k], 5, 0, 0);
      popMatrix();
    }

    //draw five circles to represent the five countries, the radius value should show the total counts
    for (int j = 0; j < 5; j++) {
      stroke(colors[j]);
      strokeWeight(2);
      noFill();
      ellipse(0, 0, radius[i][j], radius[i][j]);
    }

    // draw the books of five countries with the points and lines
    drawBooks(showChina, cBooks);
    drawBooks(showJapan, jBooks);
    drawBooks(showIndia, iBooks);
    drawBooks(showKorea, kBooks);
    drawBooks(showVietnam, vBooks);
    
    // draw the vertex curves of the book title equals the country name
    if (showCurve == true) {
    drawCurve(showChina, cBooks, colors[2], "China");
    drawCurve(showIndia, iBooks, colors[3], "India");
    drawCurve(showJapan, jBooks, colors[0], "Japan");
    drawCurve(showKorea, kBooks, colors[4], "Korea");
    drawCurve(showVietnam, vBooks, colors[1], "Vietnam");
    }
    
    textSize(9);
    fill(255);
    text (year, -8, 0, 5);
    // move to the next year
    translate (0, 0, -120);
  }
  popMatrix();

  gui();


  // when interact with cp5, keep camera still.
 /* if (cp5.getWindow().isMouseOver()) {
    cam.setActive(false);
  } else {
    cam.setActive(true);
  }*/
}
